-- functions to get bosses and loot from our tables

local _, at = ...

local loot = at["Loot"]

local function pathExists(t, k, ...)
	if k then
		if t[k] then
			return pathExists(t[k], ...)
		else
			return false
		end
	else
		return true
	end
end

local function splitBosses(list)
	local bosses = {}
	for boss in list:gmatch("([^@]+)") do
		bosses[#bosses+1] = boss
	end
	return bosses
end

local bosscache = {}
setmetatable(bosscache, {__mode = "kv"})
function at:GetBosses(instanceType, instance)
	local key = instanceType..instance
	if bosscache[key] then return bosscache[key] end
	if pathExists(loot, instanceType, instance) then
		bosscache[key] = splitBosses(loot[instanceType][instance].BossList)
		return bosscache[key]
	else
		bosscache[key] = false
		return false
	end
end

local multibosscache = {}
setmetatable(multibosscache, {__mode = "kv"})
function at:IsMultiBoss(instanceType, instance, boss)
	local key = instanceType..instance..boss
	if multibosscache[key] then return multibosscache[key] end
	if pathExists(loot, instanceType, instance, boss) and (type(loot[instanceType][instance][boss]) == "string") then
		multibosscache[key] = splitBosses(loot[instanceType][instance][boss])
		return multibosscache[key]
	else
		multibosscache[key] = false
		return false
	end
end

local function getLoot(t)
	local loot = {}
	for mode, list in pairs(t) do
		loot[mode] = {}
		for itemstring in list:gmatch("([^,]+)") do
			local id, value = itemstring:match("^([^:]+):(.+)$")
			id, value = tonumber(id) or tonumber(itemstring), tonumber(value) or true
			loot[mode][id] = value
		end
	end
	return loot
end

local function multiBossLoot(instanceType, instance, subBosses)
	local loot = {}
	for i, boss in ipairs(subBosses) do
		local temploot = getLoot(loot[instanceType][instance][boss])
		for mode, lt in pairs(temploot) do
			loot[mode] = loot[mode] or {}
			for id in pairs(lt) do -- strip percentages since they tend to be useless here
				loot[mode][id] = true
			end
		end
	end
	return loot
end

local lootCache = {}
setmetatable(lootCache, {__mode = "kv"})
function at:GetLoot(instanceType, instance, boss)
	local key = instanceType..instance..boss
	if lootCache[key] then return lootCache[key] end
	if pathExists(loot, instanceType, instance, boss) then
		local mBoss = self:IsMultiBoss(instanceType, instance, boss)
		if mBoss then
			lootCache[key] = multiBossLoot(instanceType, instance, mBoss)
			return lootCache[key]
		else
			lootCache[key] = getLoot(loot[instanceType][instance][boss])
			return lootCache[key]
		end
	else
		lootCache[key] = false
		return false
	end
end

local isBossCache = {}
setmetatable(isBossCache, {__mode = "kv"})
function at:IsBoss(instanceType, instance, boss)
	local key = instanceType..instance..boss
	if isBossCache[key] then return isBossCache[key] end
	if pathExists(loot, instanceType, instance, boss) then
		isBossCache[key] = true
		return true
	else
		isBossCache[key] = false
		return false
	end

end
