
local _, at = ...

YssInstanceMaps = CreateFrame("frame")
local YssInstanceMaps = YssInstanceMaps
YssInstanceMaps:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
function YssInstanceMaps:Print(...) print("|cFF33FF99YssInstanceMaps|r:", ...) end
YssInstanceMaps:RegisterEvent("ADDON_LOADED")

local broker = LibStub("LibDataBroker-1.1")
local icon = LibStub("LibDBIcon-1.0")

local L = LibStub("AceLocale-3.0"):GetLocale("YssInstanceMaps", true)
local BZ = LibStub("LibBabble-Zone-3.0", true):GetUnstrictLookupTable()
local BB = LibStub("LibBabble-Boss-3.0", true):GetUnstrictLookupTable()
local AceGUI = LibStub("AceGUI-3.0")

--upvalue functions and constants we use in onupdate functions
local GetCursorPosition = GetCursorPosition
local sqrt = sqrt
local sin = math.sin
local cos = math.cos
local pi = math.pi

local enableLootTooltip
local dropdowntype = nil --'type', 'zone', 'level'
local lootParent

local YssInstanceMapsFrame = CreateFrame("frame", "YssInstanceMapsFrame", UIParent)
	tinsert(UISpecialFrames,"YssInstanceMapsFrame")
	YssInstanceMapsFrame:SetFrameStrata("HIGH")
	YssInstanceMapsFrame:SetWidth(1024)
	YssInstanceMapsFrame:SetHeight(768)
	YssInstanceMapsFrame:EnableMouse(true)
	YssInstanceMapsFrame:SetMovable(true)
	YssInstanceMapsFrame:RegisterForDrag("LeftButton")
	YssInstanceMapsFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
	YssInstanceMapsFrame:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		x = self:GetLeft()
		y = self:GetTop() - (GetScreenHeight() * (UIParent:GetEffectiveScale() / self:GetEffectiveScale()))
		self:ClearAllPoints()
		self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y) -- always anchor to top left this helps us with our resizing
		YssInstanceMaps.db.profile.frame.x = x
		YssInstanceMaps.db.profile.frame.y = y
	end)

	local YssInstanceMapsTypeDropdown = CreateFrame("frame", nil, YssInstanceMapsFrame)
		YssInstanceMapsTypeDropdown:SetWidth(180)
		YssInstanceMapsTypeDropdown:SetHeight(32)
		YssInstanceMapsTypeDropdown:SetPoint("TOPRIGHT", YssInstanceMapsFrame, "TOP", -5, -35)
		YssInstanceMapsTypeDropdown.leftart = YssInstanceMapsTypeDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsTypeDropdown.leftart:SetWidth(25)
			YssInstanceMapsTypeDropdown.leftart:SetHeight(64)
			YssInstanceMapsTypeDropdown.leftart:SetPoint("TOPLEFT", 0, 17)
			YssInstanceMapsTypeDropdown.leftart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsTypeDropdown.leftart:SetTexCoord(0,0.1953125,0,1)
		YssInstanceMapsTypeDropdown.middleart = YssInstanceMapsTypeDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsTypeDropdown.middleart:SetWidth(130)
			YssInstanceMapsTypeDropdown.middleart:SetHeight(64)
			YssInstanceMapsTypeDropdown.middleart:SetPoint("LEFT", YssInstanceMapsTypeDropdown.leftart, "RIGHT")
			YssInstanceMapsTypeDropdown.middleart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsTypeDropdown.middleart:SetTexCoord(0.1953125,0.8046875,0,1)
		YssInstanceMapsTypeDropdown.rightart = YssInstanceMapsTypeDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsTypeDropdown.rightart:SetWidth(25)
			YssInstanceMapsTypeDropdown.rightart:SetHeight(64)
			YssInstanceMapsTypeDropdown.rightart:SetPoint("LEFT", YssInstanceMapsTypeDropdown.middleart, "RIGHT")
			YssInstanceMapsTypeDropdown.rightart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsTypeDropdown.rightart:SetTexCoord(0.8046875,1,0,1)
		YssInstanceMapsTypeDropdown.text = YssInstanceMapsTypeDropdown:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
			YssInstanceMapsTypeDropdown.text:SetHeight(10)
			YssInstanceMapsTypeDropdown.text:SetPoint("RIGHT", YssInstanceMapsTypeDropdown.rightart, -43, 2)
			YssInstanceMapsTypeDropdown.text:SetPoint("LEFT", YssInstanceMapsTypeDropdown.leftart, 25, 2)
			YssInstanceMapsTypeDropdown.text:SetJustifyH("RIGHT")
		YssInstanceMapsTypeDropdown.desc = YssInstanceMapsTypeDropdown:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
			YssInstanceMapsTypeDropdown.desc:SetHeight(10)
			YssInstanceMapsTypeDropdown.desc:SetPoint("TOPLEFT", 20, 10)
			YssInstanceMapsTypeDropdown.desc:SetText('Instance Type')
		YssInstanceMapsTypeDropdown.button = CreateFrame("Button", nil, YssInstanceMapsTypeDropdown)
			YssInstanceMapsTypeDropdown.button:SetWidth(24)
			YssInstanceMapsTypeDropdown.button:SetHeight(24)
			YssInstanceMapsTypeDropdown.button.type = 'type'
			YssInstanceMapsTypeDropdown.button:SetPoint("TOPRIGHT", YssInstanceMapsTypeDropdown.rightart, -16, -18)
			YssInstanceMapsTypeDropdown.button:SetNormalTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Up]])
			YssInstanceMapsTypeDropdown.button:SetPushedTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Down]])
			YssInstanceMapsTypeDropdown.button:SetHighlightTexture([[Interface\Buttons\UI-Common-MouseHilight]], "ADD")
			YssInstanceMapsTypeDropdown.button:SetDisabledTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Disabled]])

	local YssInstanceMapsZoneDropdown = CreateFrame("frame", nil, YssInstanceMapsFrame)
		YssInstanceMapsZoneDropdown:SetWidth(180)
		YssInstanceMapsZoneDropdown:SetHeight(32)
		YssInstanceMapsZoneDropdown:SetPoint("TOPLEFT", YssInstanceMapsFrame, "TOP", 5, -35)
		YssInstanceMapsZoneDropdown.leftart = YssInstanceMapsZoneDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsZoneDropdown.leftart:SetWidth(25)
			YssInstanceMapsZoneDropdown.leftart:SetHeight(64)
			YssInstanceMapsZoneDropdown.leftart:SetPoint("TOPLEFT", 0, 17)
			YssInstanceMapsZoneDropdown.leftart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsZoneDropdown.leftart:SetTexCoord(0,0.1953125,0,1)
		YssInstanceMapsZoneDropdown.middleart = YssInstanceMapsZoneDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsZoneDropdown.middleart:SetWidth(130)
			YssInstanceMapsZoneDropdown.middleart:SetHeight(64)
			YssInstanceMapsZoneDropdown.middleart:SetPoint("LEFT", YssInstanceMapsZoneDropdown.leftart, "RIGHT")
			YssInstanceMapsZoneDropdown.middleart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsZoneDropdown.middleart:SetTexCoord(0.1953125,0.8046875,0,1)
		YssInstanceMapsZoneDropdown.rightart = YssInstanceMapsZoneDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsZoneDropdown.rightart:SetWidth(25)
			YssInstanceMapsZoneDropdown.rightart:SetHeight(64)
			YssInstanceMapsZoneDropdown.rightart:SetPoint("LEFT", YssInstanceMapsZoneDropdown.middleart, "RIGHT")
			YssInstanceMapsZoneDropdown.rightart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsZoneDropdown.rightart:SetTexCoord(0.8046875,1,0,1)
		YssInstanceMapsZoneDropdown.text = YssInstanceMapsZoneDropdown:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
			YssInstanceMapsZoneDropdown.text:SetHeight(10)
			YssInstanceMapsZoneDropdown.text:SetPoint("RIGHT", YssInstanceMapsZoneDropdown.rightart, -43, 2)
			YssInstanceMapsZoneDropdown.text:SetPoint("LEFT", YssInstanceMapsZoneDropdown.leftart, 25, 2)
			YssInstanceMapsZoneDropdown.text:SetJustifyH("RIGHT")
		YssInstanceMapsZoneDropdown.desc = YssInstanceMapsZoneDropdown:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
			YssInstanceMapsZoneDropdown.desc:SetHeight(10)
			YssInstanceMapsZoneDropdown.desc:SetPoint("TOPLEFT", 20, 10)
			YssInstanceMapsZoneDropdown.desc:SetText(ZONE)
		YssInstanceMapsZoneDropdown.button = CreateFrame("Button", nil, YssInstanceMapsZoneDropdown)
			YssInstanceMapsZoneDropdown.button:SetWidth(24)
			YssInstanceMapsZoneDropdown.button:SetHeight(24)
			YssInstanceMapsZoneDropdown.button.type = 'zone'
			YssInstanceMapsZoneDropdown.button:SetPoint("TOPRIGHT", YssInstanceMapsZoneDropdown.rightart, -16, -18)
			YssInstanceMapsZoneDropdown.button:SetNormalTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Up]])
			YssInstanceMapsZoneDropdown.button:SetPushedTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Down]])
			YssInstanceMapsZoneDropdown.button:SetHighlightTexture([[Interface\Buttons\UI-Common-MouseHilight]], "ADD")
			YssInstanceMapsZoneDropdown.button:SetDisabledTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Disabled]])

	local YssInstanceMapsLevelDropdown = CreateFrame("frame", nil, YssInstanceMapsFrame)
		YssInstanceMapsLevelDropdown:SetWidth(180)
		YssInstanceMapsLevelDropdown:SetHeight(32)
		YssInstanceMapsLevelDropdown:SetPoint("TOPRIGHT", YssInstanceMapsFrame, "TOPRIGHT", -20, -35)
		YssInstanceMapsLevelDropdown.leftart = YssInstanceMapsLevelDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsLevelDropdown.leftart:SetWidth(25)
			YssInstanceMapsLevelDropdown.leftart:SetHeight(64)
			YssInstanceMapsLevelDropdown.leftart:SetPoint("TOPLEFT", 0, 17)
			YssInstanceMapsLevelDropdown.leftart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsLevelDropdown.leftart:SetTexCoord(0,0.1953125,0,1)
		YssInstanceMapsLevelDropdown.middleart = YssInstanceMapsLevelDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsLevelDropdown.middleart:SetWidth(130)
			YssInstanceMapsLevelDropdown.middleart:SetHeight(64)
			YssInstanceMapsLevelDropdown.middleart:SetPoint("LEFT", YssInstanceMapsLevelDropdown.leftart, "RIGHT")
			YssInstanceMapsLevelDropdown.middleart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsLevelDropdown.middleart:SetTexCoord(0.1953125,0.8046875,0,1)
		YssInstanceMapsLevelDropdown.rightart = YssInstanceMapsLevelDropdown:CreateTexture(nil, 'ARTWORK')
			YssInstanceMapsLevelDropdown.rightart:SetWidth(25)
			YssInstanceMapsLevelDropdown.rightart:SetHeight(64)
			YssInstanceMapsLevelDropdown.rightart:SetPoint("LEFT", YssInstanceMapsLevelDropdown.middleart, "RIGHT")
			YssInstanceMapsLevelDropdown.rightart:SetTexture([[Interface\Glues\CharacterCreate\CharacterCreate-LabelFrame]])
			YssInstanceMapsLevelDropdown.rightart:SetTexCoord(0.8046875,1,0,1)
		YssInstanceMapsLevelDropdown.text = YssInstanceMapsLevelDropdown:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
			YssInstanceMapsLevelDropdown.text:SetHeight(10)
			YssInstanceMapsLevelDropdown.text:SetPoint("RIGHT", YssInstanceMapsLevelDropdown.rightart, -43, 2)
			YssInstanceMapsLevelDropdown.text:SetPoint("LEFT", YssInstanceMapsLevelDropdown.leftart, 25, 2)
			YssInstanceMapsLevelDropdown.text:SetJustifyH("RIGHT")
		YssInstanceMapsLevelDropdown.desc = YssInstanceMapsLevelDropdown:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
			YssInstanceMapsLevelDropdown.desc:SetHeight(10)
			YssInstanceMapsLevelDropdown.desc:SetPoint("TOPLEFT", 20, 10)
			YssInstanceMapsLevelDropdown.desc:SetText(FLOOR)
		YssInstanceMapsLevelDropdown.button = CreateFrame("Button", nil, YssInstanceMapsLevelDropdown)
			YssInstanceMapsLevelDropdown.button:SetWidth(24)
			YssInstanceMapsLevelDropdown.button:SetHeight(24)
			YssInstanceMapsLevelDropdown.button.type = 'level'
			YssInstanceMapsLevelDropdown.button:SetPoint("TOPRIGHT", YssInstanceMapsLevelDropdown.rightart, -16, -18)
			YssInstanceMapsLevelDropdown.button:SetNormalTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Up]])
			YssInstanceMapsLevelDropdown.button:SetPushedTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Down]])
			YssInstanceMapsLevelDropdown.button:SetHighlightTexture([[Interface\Buttons\UI-Common-MouseHilight]], "ADD")
			YssInstanceMapsLevelDropdown.button:SetDisabledTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Disabled]])

	local BGT1 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGT1:SetWidth(256)
		BGT1:SetHeight(256)
		BGT1:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", -512, 384)
		BGT1:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Top1")
	local BGT2 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGT2:SetWidth(256)
		BGT2:SetHeight(256)
		BGT2:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", -256, 384)
		BGT2:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Top2")
	local BGT3 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGT3:SetWidth(256)
		BGT3:SetHeight(256)
		BGT3:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", 0, 384)
		BGT3:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Top3")
	local BGT4 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGT4:SetWidth(256)
		BGT4:SetHeight(256)
		BGT4:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", 256, 384)
		BGT4:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Top4")
	local BGM1 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGM1:SetWidth(256)
		BGM1:SetHeight(256)
		BGM1:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", -512, 128)
		BGM1:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Middle1")
	local BGM2 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGM2:SetWidth(256)
		BGM2:SetHeight(256)
		BGM2:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", -256, 128)
		BGM2:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Middle2")
	local BGM3 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGM3:SetWidth(256)
		BGM3:SetHeight(256)
		BGM3:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", 0, 128)
		BGM3:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Middle3")
	local BGM4 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGM4:SetWidth(256)
		BGM4:SetHeight(256)
		BGM4:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", 256, 128)
		BGM4:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Middle4")
	local BGB1 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGB1:SetWidth(256)
		BGB1:SetHeight(256)
		BGB1:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", -512, -128)
		BGB1:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Bottom1")
	local BGB2 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGB2:SetWidth(256)
		BGB2:SetHeight(256)
		BGB2:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", -256, -128)
		BGB2:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Bottom2")
	local BGB3 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGB3:SetWidth(256)
		BGB3:SetHeight(256)
		BGB3:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", 0, -128)
		BGB3:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Bottom3")
	local BGB4 = YssInstanceMapsFrame:CreateTexture(nil, 'BACKGROUND')
		BGB4:SetWidth(256)
		BGB4:SetHeight(256)
		BGB4:SetPoint("TOPLEFT", YssInstanceMapsFrame, "CENTER", 256, -128)
		BGB4:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\UI-WorldMap-Bottom4")
	local cursorCoords = YssInstanceMapsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		cursorCoords:SetPoint("RIGHT", YssInstanceMapsFrame, "CENTER", 0, -367)

local YssInstanceMapsFrameCloseButton = CreateFrame("Button", nil, YssInstanceMapsFrame)
	YssInstanceMapsFrameCloseButton:SetWidth(32)
	YssInstanceMapsFrameCloseButton:SetHeight(32)
	YssInstanceMapsFrameCloseButton:SetNormalTexture([[Interface\Buttons\UI-Panel-MinimizeButton-Up]])
	YssInstanceMapsFrameCloseButton:SetPushedTexture([[Interface\Buttons\UI-Panel-MinimizeButton-Down]])
	YssInstanceMapsFrameCloseButton:SetHighlightTexture([[Interface\Buttons\UI-Panel-MinimizeButton-Highlight]], 'ADD')
	YssInstanceMapsFrameCloseButton:SetPoint('TOPRIGHT', YssInstanceMapsFrame, 'TOP', 516,4)
	YssInstanceMapsFrameCloseButton:SetScript('OnClick', function() YssInstanceMapsFrame:Hide() end)

local scaler = CreateFrame("Frame", nil, YssInstanceMapsFrame)
	scaler:SetPoint("BOTTOMRIGHT",YssInstanceMapsFrame,"BOTTOMRIGHT",0,0)
	scaler:SetWidth(28)
	scaler:SetHeight(28)
	scaler:EnableMouse()

	scaler.Texture = scaler:CreateTexture(nil, "BACKGROUND")
		scaler.Texture:SetWidth(20)
		scaler.Texture:SetHeight(20)
		scaler.Texture:SetPoint("BOTTOMRIGHT", -4, 4)
		scaler.Texture:SetTexture([[Interface\BUTTONS\UI-AutoCastableOverlay]])
		scaler.Texture:SetTexCoord(0.619, 0.760, 0.612, 0.762)
		scaler.Texture:SetDesaturated(true)

	local mousetracker = CreateFrame("Frame", nil ,UIParent)
		mousetracker:SetFrameStrata("TOOLTIP")
		mousetracker:SetAllPoints(scaler)
		mousetracker:EnableMouse(true)
		mousetracker:SetScript("OnEnter", function()
			scaler.Texture:SetDesaturated(false)
		end)
		mousetracker:SetScript("OnLeave", function()
			scaler.Texture:SetDesaturated(true)
		end)

		local ScalerOriginalState = {
			dist = 0,
			width = 0,
			height = 0,
			left = 0,
			top = 0,
			scale = 1,
		}
		local function GetScaleDistance() -- distance from cursor to TopLeft :)
			local left, top = ScalerOriginalState.left, ScalerOriginalState.top
			local scale = ScalerOriginalState.EFscale

			local x, y = GetCursorPosition()
			local cx = x/scale - left
			local cy = top - y/scale

			return sqrt(cx*cx+cy*cy)
		end

		mousetracker:SetScript("OnMouseUp", function(self)
				self:SetScript("OnUpdate", nil)
				self:SetAllPoints(scaler)
			end)
		mousetracker:SetScript("OnMouseDown",function(self)
				ScalerOriginalState.left, ScalerOriginalState.top = YssInstanceMapsFrame:GetLeft(), YssInstanceMapsFrame:GetTop()
				ScalerOriginalState.cx, ScalerOriginalState.cy = select(4, YssInstanceMapsFrame:GetPoint())
				ScalerOriginalState.EFscale = YssInstanceMapsFrame:GetEffectiveScale()
				ScalerOriginalState.scale = YssInstanceMapsFrame:GetScale()
				ScalerOriginalState.dist = GetScaleDistance()
				self:SetScript("OnUpdate", function(self)
						local scale = GetScaleDistance()/ScalerOriginalState.dist*ScalerOriginalState.scale
						if scale < .2 then scale = .2
						elseif scale > 1 then scale = 1
						end
						YssInstanceMapsFrame:SetScale(scale)

						local s = ScalerOriginalState.scale/YssInstanceMapsFrame:GetScale()
						local x = ScalerOriginalState.cx*s
						local y = ScalerOriginalState.cy*s
						YssInstanceMapsFrame:ClearAllPoints()
						YssInstanceMapsFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
						YssInstanceMaps.db.profile.frame.scale = scale
						YssInstanceMaps.db.profile.frame.x = x
						YssInstanceMaps.db.profile.frame.y = y
					end)
				self:SetAllPoints(UIParent)
			end)
	local YssInstanceMapsDetailFrame = CreateFrame('Frame', 'YssInstanceMapsDetailFrame', YssInstanceMapsFrame)
		YssInstanceMapsDetailFrame:SetWidth(1002)
		YssInstanceMapsDetailFrame:SetHeight(668)
		YssInstanceMapsDetailFrame:EnableMouse(true)
		YssInstanceMapsDetailFrame:SetPoint('TOPLEFT', YssInstanceMapsFrame, 'TOP', -502, -69)
		YssInstanceMapsDetailFrame.Tiles = {}

		YssInstanceMapsDetailFrame.Tiles[1] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[1]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[1]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[1]:SetPoint("TOPLEFT")
		YssInstanceMapsDetailFrame.Tiles[2] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[2]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[2]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[2]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[1], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[3] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[3]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[3]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[3]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[2], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[4] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[4]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[4]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[4]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[3], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[5] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[5]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[5]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[5]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[1], "BOTTOMLEFT")
		YssInstanceMapsDetailFrame.Tiles[6] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[6]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[6]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[6]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[5], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[7] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[7]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[7]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[7]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[6], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[8] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[8]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[8]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[8]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[7], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[9] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[9]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[9]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[9]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[5], "BOTTOMLEFT")
		YssInstanceMapsDetailFrame.Tiles[10] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[10]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[10]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[10]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[9], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[11] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[11]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[11]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[11]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[10], "TOPRIGHT")
		YssInstanceMapsDetailFrame.Tiles[12] = YssInstanceMapsDetailFrame:CreateTexture(nil, 'BACKGROUND')
			YssInstanceMapsDetailFrame.Tiles[12]:SetWidth(256)
			YssInstanceMapsDetailFrame.Tiles[12]:SetHeight(256)
			YssInstanceMapsDetailFrame.Tiles[12]:SetPoint("TOPLEFT", YssInstanceMapsDetailFrame.Tiles[11], "TOPRIGHT")

		YssInstanceMapsDetailFrame:SetScript('OnUpdate', function(self)
				local left, top = self:GetLeft(), self:GetTop()
				local width, height = self:GetWidth(), self:GetHeight()
				local scale = self:GetEffectiveScale()

				local x, y = GetCursorPosition()
				x = (x/scale - left) / width
				y = (top - y/scale) / height

				if x < 0 or x > 1 or y < 0 or y > 1 then
					return cursorCoords:SetText('')
				end
				cursorCoords:SetFormattedText("%.2f, %.2f", 100 * x, 100 * y)
		end)

	local YssInstanceMapsLootFrame = CreateFrame("frame", "YssInstanceMapsLootFrame", YssInstanceMapsDetailFrame)
		YssInstanceMapsLootFrame:SetFrameStrata("DIALOG")
		YssInstanceMapsLootFrame:EnableMouse(true)
		YssInstanceMapsLootFrame:SetWidth(1002)
		YssInstanceMapsLootFrame:SetHeight(668)
		YssInstanceMapsLootFrame:SetScale(.1)
		YssInstanceMapsLootFrame:SetPoint("CENTER", YssInstanceMapsDetailFrame, "CENTER")
		YssInstanceMapsLootFrame.frames = {} -- put our lootframes here
		YssInstanceMapsLootFrame.contentframes = {} -- put the content widget here so we can drop in our LootWidgets
		YssInstanceMapsLootFrame:SetScript("OnHide", function(self)
			for i, cF in ipairs(self.contentframes) do
				cF:ReleaseChildren()
			end
		end)
		YssInstanceMapsLootFrame.bg = YssInstanceMapsLootFrame:CreateTexture()
			YssInstanceMapsLootFrame.bg:SetTexture(0,0,0,0.75)
			YssInstanceMapsLootFrame.bg:SetAllPoints(YssInstanceMapsLootFrame)
		YssInstanceMapsLootFrame:Hide()
		YssInstanceMapsLootFrame.hideButton = CreateFrame("Button","YssInstanceMapsHideLootButton",YssInstanceMapsLootFrame,"UIPanelButtonTemplate2")
			YssInstanceMapsLootFrame.hideButton:SetWidth(120)
			YssInstanceMapsLootFrame.hideButton:SetHeight(24)
			YssInstanceMapsLootFrame.hideButton:SetPoint("BOTTOM", YssInstanceMapsLootFrame, "BOTTOM", 0, 5)
			YssInstanceMapsLootFrame.hideButton:SetText("Hide Loot")
			YssInstanceMapsLootFrame.hideButton:SetScript("OnClick", function()
				YssInstanceMapsLootFrame.LootHide:Play()
			end)
		YssInstanceMapsLootFrame.BossName = YssInstanceMapsLootFrame:CreateFontString(nil, "OVERLAY")
			YssInstanceMapsLootFrame.BossName:SetFont([[Fonts\FRIZQT__.TTF]], 30, "OUTLINE")
			YssInstanceMapsLootFrame.BossName:SetPoint("TOP", YssInstanceMapsLootFrame, "TOP",0,-10)
			YssInstanceMapsLootFrame.BossName:SetText("Boss Name Here")

		local ToggleLootShow = YssInstanceMapsLootFrame:CreateAnimationGroup()
			local LootShow = ToggleLootShow:CreateAnimation("Animation")
				LootShow:SetDuration(.45)
				LootShow:SetOrder(1)
				LootShow:SetScript("OnPlay", function(self)
					enableLootTooltip = false
					YssInstanceMapsLootFrame:SetScale(.05)
					YssInstanceMapsLootFrame:Show()
					YssInstanceMapsLootFrame:SetAlpha(0)
				end)
				LootShow:SetScript("OnUpdate", function(self)
					local progress = self:GetProgress()
					local scale = (progress+.05)/1.05 -- ensures that our scale will never == 0
					local x = (YssInstanceMapsLootFrame.Xor/scale)*(1-progress)
					local y = (YssInstanceMapsLootFrame.Yor/scale)*(1-progress)
					YssInstanceMapsLootFrame:ClearAllPoints()
					YssInstanceMapsLootFrame:SetScale(scale)
					YssInstanceMapsLootFrame:SetAlpha(progress^(1/2))
					YssInstanceMapsLootFrame:SetPoint("CENTER", lootParent, "CENTER", x, y)
				end)
				LootShow:SetScript("OnFinished", function(self)
					YssInstanceMapsLootFrame:ClearAllPoints()
					YssInstanceMapsLootFrame:SetScale(1)
					enableLootTooltip = true
					YssInstanceMapsLootFrame:SetAlpha(1)
					YssInstanceMapsLootFrame:SetPoint("CENTER", lootParent, "CENTER")
				end)
		ToggleLootShow.LootShow = LootShow
		ToggleLootShow:SetLooping("NONE")
		YssInstanceMapsLootFrame.ToggleLootShow = ToggleLootShow
		YssInstanceMapsLootFrame.LootShow = LootShow
		local ToggleLootHide = YssInstanceMapsLootFrame:CreateAnimationGroup()
			local LootHide = ToggleLootHide:CreateAnimation("Animation")
				LootHide:SetScript("OnPlay", function(self)
					enableLootTooltip = false
					GameTooltip:Hide()
					YssInstanceMapsLootFrame:SetAlpha(1)
				end)
				LootHide:SetDuration(.45)
				LootHide:SetOrder(1)
				LootHide:SetScript("OnUpdate", function(self)
					local progress = self:GetProgress()
					local scale = ((1-progress)+.05)/1.05 -- ensures that our scale will never == 0
					local x = (YssInstanceMapsLootFrame.Xor/scale)*progress
					local y = (YssInstanceMapsLootFrame.Yor/scale)*progress
					YssInstanceMapsLootFrame:SetAlpha((1-progress)^(1/2))
					YssInstanceMapsLootFrame:ClearAllPoints()
					YssInstanceMapsLootFrame:SetScale(scale)
					YssInstanceMapsLootFrame:SetPoint("CENTER", lootParent, "CENTER", x, y)
				end)
				LootHide:SetScript("OnFinished", function(self)
					YssInstanceMapsLootFrame:Hide()
					YssInstanceMapsLootFrame:SetAlpha(1)
				end)
		ToggleLootHide.LootHide = LootHide
		ToggleLootHide:SetLooping("NONE")
		YssInstanceMapsLootFrame.ToggleLootHide = ToggleLootHide
		YssInstanceMapsLootFrame.LootHide = LootHide


local function setupRotationTexture(texture, duration)
	texture:SetPoint("CENTER", YssInstanceMapsDetailFrame)
	texture:SetBlendMode("ADD")
	texture:SetAlpha(.5)
	local animGroup = texture:CreateAnimationGroup()
	local Rotation = animGroup:CreateAnimation("Rotation")
		animGroup:SetLooping("REPEAT")
		Rotation:SetDuration(duration)
		Rotation:SetOrder(1)
		Rotation:SetDegrees(-360)
		Rotation:SetOrigin("CENTER", 0, 0)
		animGroup.Rotation = Rotation
	animGroup:Play()
	return animGroup
end

local Swirl_anim = {}
	Swirl_anim[1] = YssInstanceMapsFrame:CreateTexture(nil, 'ARTWORK')
		Swirl_anim[1]:SetWidth(600)
		Swirl_anim[1]:SetHeight(600)
		Swirl_anim[1]:SetTexture([[Interface\Addons\YssInstanceMaps\Art\twirl]])
		Swirl_anim[1].anim = setupRotationTexture(Swirl_anim[1], 8)
	Swirl_anim[2] = YssInstanceMapsFrame:CreateTexture(nil, 'ARTWORK')
		Swirl_anim[2]:SetWidth(350)
		Swirl_anim[2]:SetHeight(350)
		Swirl_anim[2]:SetTexture([[Interface\Addons\YssInstanceMaps\Art\twirl]])
		Swirl_anim[2].anim = setupRotationTexture(Swirl_anim[2], 6)
	Swirl_anim[3] = YssInstanceMapsFrame:CreateTexture(nil, 'ARTWORK')
		Swirl_anim[3]:SetWidth(200)
		Swirl_anim[3]:SetHeight(200)
		Swirl_anim[3]:SetTexture([[Interface\Addons\YssInstanceMaps\Art\twirl]])
		Swirl_anim[3].anim = setupRotationTexture(Swirl_anim[3], 4)

function YssInstanceMaps:ADDON_LOADED(event, addon)
	if addon == "YssInstanceMaps" then
		self:UnregisterEvent("ADDON_LOADED")
		self.ADDON_LOADED = nil
		if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
	end
end

function YssInstanceMaps:PLAYER_LOGIN(event, addon)
	local defaults = {
		['profile'] = {
			maptype = 'Dungeons',
			--zone = "Ahn'kahet: The Old Kingdom",
			--level = 1,
			frame = {
				scale = .8,
				x = 30,
				y = -30,
			},
			['LibDBIcon'] = {
				['hide'] = true,
			},
		},
	}
	self.db = LibStub("AceDB-3.0"):New("YssInstanceMapsDB", defaults)

	self:RegisterEvent('COMPANION_LEARNED')
	self:COMPANION_LEARNED()

	YssInstanceMapsFrame:ClearAllPoints()
	YssInstanceMapsFrame:SetScale(self.db.profile.frame.scale)
	YssInstanceMapsFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.frame.x, self.db.profile.frame.y)

	local baselevel, numlevels = self:GetLowestLevel(self.db.profile.maptype, self.db.profile.zone)

	YssInstanceMapsTypeDropdown.text:SetText(YssInstanceMaps.MapTypes[YssInstanceMaps.db.profile.maptype])
	if baselevel then
		YssInstanceMapsZoneDropdown.text:SetText(BZ[YssInstanceMaps.db.profile.zone])
		self:SetMap(self.Maps[self.db.profile.maptype][self.db.profile.zone], self.db.profile.level)
		self:AddBosses(self.db.profile.maptype, self.db.profile.zone, self.db.profile.level, YssInstanceMapsDetailFrame)
		if numlevels then
			YssInstanceMapsLevelDropdown:Show()
			YssInstanceMapsLevelDropdown.text:SetText(self:GetLevelName(self.db.profile.maptype, self.db.profile.zone, self.db.profile.level))
		else
			YssInstanceMapsLevelDropdown:Hide()
		end
	else
		self.db.profile.zone = nil
		self.db.profile.level = nil
		YssInstanceMapsLevelDropdown:Hide()
	end

	self.obj = LibStub("LibDataBroker-1.1"):NewDataObject("YssInstanceMaps", {
		icon = "Interface\\ICONS\\INV_Misc_Head_Dragon_Red",
		label = "|cFF33FF99YssInstanceMaps|r",
		type = "launcher",
		text  = "YssInstanceMaps",
		OnClick = function(clickedFrame, button)
			if YssInstanceMapsFrame:IsShown() then
				YssInstanceMapsFrame:Hide()
			else
				YssInstanceMapsFrame:Show()
			end
		end,
		OnTooltipShow = function(tt)
			tt:AddLine('YssInstanceMaps')
			tt:AddLine("|cffffff00Click|r to toggle YssInstanceMaps")
			--tt:AddLine("|cffffff00Right-click|r to open the options menu")
		end,
	})
	icon:Register("YssInstanceMaps", YssInstanceMaps.obj, YssInstanceMaps.db.profile.LibDBIcon)

	SLASH_YssInstanceMaps1 = "/YssInstanceMaps";
	SLASH_YssInstanceMaps2 = "/YIM";
	SlashCmdList["YssInstanceMaps"] = function()
		YssInstanceMapsFrame:Show()
	end

	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

local cid = 0
function YssInstanceMaps:COMPANION_LEARNED()
	for id = 1,GetNumCompanions("CRITTER") do
		if 29147 == GetCompanionInfo("CRITTER", id) then
			cid = 29147
			CallCompanion("CRITTER", id)
			self:UnregisterEvent('COMPANION_LEARNED')
			break
		end
	end
end

local WorldMapDetailFrame = _G['WorldMapDetailFrame']
local origSetTex = WorldMapDetailTile1.SetTexture
function WorldMapDetailTile1.SetTexture(self, tex, ...)
	origSetTex(self, tex, ...)
	YssInstanceMaps:ClearBosses(WorldMapDetailFrame)
	if not tex then return end
	local map , level = string.match(tex, "([^\\]+)(%d+)_%d+")
	if not map then
		map = string.match(tex, "([^\\]+)%d+")
		level = 0
	end
	level = tonumber(level)
	local aMap
	local mapType
	local temp
	if UnitInRaid('player') and YssInstanceMaps.Maps.rRaid[map] then
		temp =  YssInstanceMaps.Levels.Raid[YssInstanceMaps.Maps.rRaid[map]]
		if type(temp) == 'table' then
			for i, l in ipairs(temp) do
				if l == tonumber(level) then
					aMap = YssInstanceMaps.Maps.rRaid[map]
					mapType = 'Raid'
					break
				end
			end
		elseif temp and temp <= tonumber(level) then
			aMap = YssInstanceMaps.Maps.rRaid[map]
			mapType = 'Raid'
		end
	end
	local mapTypes = {'Dungeon', 'Raid', 'Battlegrounds'}
	if not aMap then
		for i, t in ipairs(mapTypes) do
			temp =  YssInstanceMaps.Levels[t][YssInstanceMaps.Maps['r'..t][map]]

			if type(temp) == 'table' then
				for i, l in ipairs(temp) do
					if l == tonumber(level) then
						aMap = YssInstanceMaps.Maps['r'..t][map]
						mapType = t
						break
					end
				end
			elseif temp and temp >= tonumber(level) then
				aMap = YssInstanceMaps.Maps['r'..t][map]
				mapType = t
			end
			if aMap then
				break
			end
		end
	end
	if aMap then
		YssInstanceMaps:AddBosses(mapType, aMap, level, WorldMapDetailFrame)
	end
end

local instanceDifficulties = {
	Dungeon = {
		[1] = L['Normal Loot'],
		[2] = L['Heroic Loot'],
	},
	Raid = {
		[1] = L['Normal 10-man Loot'],
		[2] = L['Normal 25-man Loot'],
		[3] = L['Heroic 10-man Loot'],
		[4] = L['Heroic 25-man Loot'],
	},
	H = L['%s Hard Mode'],
	Fallback = L['Loot'],
}

function YssInstanceMaps:GetLoot(instanceType, instance, boss) --need to adjust this when PT gets adjusted for 4 types of raid loot
	local lootDesc = {}
	local numLootTables = 0
	if at:IsMultiBoss(instanceType, instance, boss) then
		error('still need to account for multi bosses')
	else
		loot = at:GetLoot(instanceType, instance, boss)
		if loot then
			for i in pairs(loot) do
				numLootTables = numLootTables + 1
				lootDesc[i] = instanceDifficulties[instanceType][i]
			end
		end
	end
	return loot, lootDesc, numLootTables
end

--/script YssInstanceMaps:SetMap('CoTStratholme', 1)
function YssInstanceMaps:SetMap(mapFileName, dungeonLevel)
	if mapFileName then
		local completeMapFileName
		if ( dungeonLevel > 0 ) then
			completeMapFileName = mapFileName..dungeonLevel.."_"
		else
			completeMapFileName = mapFileName
		end
		for i=1, NUM_WORLDMAP_DETAIL_TILES do
			YssInstanceMapsDetailFrame.Tiles[i]:SetTexture("Interface\\WorldMap\\"..mapFileName.."\\"..completeMapFileName..i)
		end
	else
		for i=1, NUM_WORLDMAP_DETAIL_TILES do
			YssInstanceMapsDetailFrame.Tiles[i]:SetTexture(nil)
		end
	end
	self:ClearBosses(YssInstanceMapsDetailFrame)
	enableLootTooltip = false
	YssInstanceMapsLootFrame:Hide()
end

function YssInstanceMaps:GetLowestLevel(MapType, Map) -- returns base level and how many total levels
	if not MapType or not Map then return end
	local levels = YssInstanceMaps.Levels[MapType][Map]
	if type(levels) == 'table' then
		return levels[1], #levels
	elseif levels > 0 then
		if levels == 1 then
			return 1, false
		else
			return 1, levels
		end
	elseif levels then
		return 0, false
	end
end

function YssInstanceMaps:GetLevelName(MapType, Map, level)
	local levelname = _G['DUNGEON_FLOOR_'..string.upper(self.Maps[MapType][Map])..level]
	if levelname then
		return levelname
	else
		if type(self.Levels[MapType][Map]) == 'table' then
			for i,l in ipairs(self.Levels[MapType][Map]) do
				if l == level then
					return string.format(FLOOR_NUMBER, i)
				end
			end
		else
			return string.format(FLOOR_NUMBER, level)
		end
	end
end

--[==[
lua> for l,x,y in string.gmatch("1:34.50:34.23|2:45.62:23.54|true", "([^|]+):([^|]+):([^|]+)") do print(l..' '..x..' '..y) end
lua> for b in string.gmatch("1:34.50:34.23|2:45.62:23.54|true", "|([^|]+)$") do print(l..' '..x..' '..y) end
]==]--

local bossframecache = {}
local totalFramesGenerated = 0
local bossframesize = 35
--local bossframesize = 150
local bossfontsize = 12

local function BossFrameOnUpdate(self)
	if self.onupdate > 0 then
		self:SetModelScale(9.2479419708252)
		self:SetPosition(0,0,-16.321418762207)
		self:SetFacing(.1)
		self.rotateBGGroup:Play()
		self:SetScript('OnUpdate', nil)
	else
		self:SetCreature(cid)
	end
	self.onupdate = self.onupdate + 1
end

local function BossFrameOnShow(self)
	self:ClearModel()
	self:SetCreature(cid)
	self:SetModelScale(1)
	self:SetPosition(0,0,0)
	self:SetFacing(0)
	self.onupdate = 0
	self:SetScript('OnUpdate', BossFrameOnUpdate)
end

local function CreateLootFrame()
	local widget = {}
	widget.frame = CreateFrame("frame", nil, YssInstanceMapsLootFrame)
	widget.LootText = widget.frame:CreateFontString(nil, "OVERLAY")
		widget.LootText:SetFont([[Fonts\FRIZQT__.TTF]], 20, "OUTLINE")
		widget.LootText:SetPoint("TOPLEFT", widget.frame, "TOPLEFT")
		widget.LootText:SetPoint("TOPRIGHT", widget.frame, "TOPRIGHT")
		widget.LootText:SetHeight(24)
		widget.LootText:SetText("LootType here")
	widget.content = CreateFrame('frame', nil, widget.frame)
		widget.content:SetPoint("TOPLEFT", widget.frame, "TOPLEFT",0,-30)
		widget.content:SetPoint("BOTTOMRIGHT", widget.frame, "BOTTOMRIGHT")
	AceGUI:RegisterAsContainer(widget)
	widget:SetLayout("fill")
	local parent = AceGUI:Create("ScrollFrame")
		parent:SetLayout("flow")
		parent.width = "fill"
		parent.height = "fill"
	widget:AddChild(parent)
	return widget, parent
end

local function LayoutLootFrames(widgets, numLootSets)
	if numLootSets == 1 then
		widgets[1].frame:SetPoint("TOP", YssInstanceMapsLootFrame, "TOP", 0, -40)
		widgets[1].frame:SetPoint("BOTTOM", YssInstanceMapsLootFrame, "BOTTOM", 0, 35)
		widgets[1].frame:SetWidth(250)
	elseif numLootSets == 2 then
		widgets[1].frame:SetPoint("TOPRIGHT", YssInstanceMapsLootFrame, "TOP", -10, -40)
		widgets[1].frame:SetPoint("BOTTOMRIGHT", YssInstanceMapsLootFrame, "BOTTOM", -10, 35)
		widgets[1].frame:SetWidth(250)

		widgets[2].frame:SetPoint("TOPLEFT", YssInstanceMapsLootFrame, "TOP", 10, -40)
		widgets[2].frame:SetPoint("BOTTOMLEFT", YssInstanceMapsLootFrame, "BOTTOM", 10, 35)
		widgets[2].frame:SetWidth(250)
	elseif numLootSets > 2 then --668, 299
		widgets[1].frame:SetPoint("TOPRIGHT", YssInstanceMapsLootFrame, "TOP", -10, -40)
		widgets[1].frame:SetPoint("BOTTOMRIGHT", YssInstanceMapsLootFrame, "BOTTOM", -10, 35+299+10)
		widgets[1].frame:SetWidth(250)

		widgets[2].frame:SetPoint("TOPLEFT", YssInstanceMapsLootFrame, "TOP", 10, -40)
		widgets[2].frame:SetPoint("BOTTOMLEFT", YssInstanceMapsLootFrame, "BOTTOM", 10, 35+299+10)
		widgets[2].frame:SetWidth(250)

		widgets[3].frame:SetPoint("TOP", widgets[1].frame, "BOTTOM", 0, -20)
		widgets[3].frame:SetPoint("BOTTOMRIGHT", YssInstanceMapsLootFrame, "BOTTOM", -10, 35)
		widgets[3].frame:SetWidth(250)
	end
	if numLootSets == 4 then
		widgets[4].frame:SetPoint("TOP", widgets[2].frame, "BOTTOM", 0, -20)
		widgets[4].frame:SetPoint("BOTTOMLEFT", YssInstanceMapsLootFrame, "BOTTOM", 10, 35)
		widgets[4].frame:SetWidth(250)
	end
end

local function LootClickHandler(self, event, link)
	if link then
		HandleModifiedItemClick(link)
	end
end

local function LootEnterHandler(self, event, button, link)
	if enableLootTooltip and link then
		GameTooltip:SetOwner(button, "ANCHOR_RIGHT", button:GetWidth()/-2, 5)
		GameTooltip:SetHyperlink(link)
	end
end

local function LootLeaveHandler()
	if enableLootTooltip then
		GameTooltip:Hide()
	end
end



local orderedLoot = {}
local function BossFrameOnMouseUp(self, button)
	lootParent = self:GetParent()
	YssInstanceMapsLootFrame:SetParent(lootParent)
	YssInstanceMapsLootFrame:SetFrameStrata("FULLSCREEN_DIALOG")
	local p,_,_,Xor, Yor = self:GetPoint()
	if p == "RIGHT" then
		Xor = 1002 + Xor - (bossframesize/2)
		Yor = -668 / 2
	end
	YssInstanceMapsLootFrame.Xor = Xor - (1002/2)
	YssInstanceMapsLootFrame.Yor = Yor + (668/2)

	YssInstanceMapsLootFrame.BossName:SetText(self.Boss)
	local loot, lootDesc, numLootTables = YssInstanceMaps:GetLoot(self.InstanceType, self.Instance, self.Boss)

	for i=1,numLootTables do
		if not YssInstanceMapsLootFrame.frames[i] then
			YssInstanceMapsLootFrame.frames[i], YssInstanceMapsLootFrame.contentframes[i] = CreateLootFrame()
		end
		YssInstanceMapsLootFrame.frames[i].LootText:SetText(lootDesc[i])
		YssInstanceMapsLootFrame.frames[i].frame:Show()
		wipe(orderedLoot)
		for item in pairs(loot[i]) do
			if item ~= 'set' then
				orderedLoot[#orderedLoot+1] = item
			end
		end
		table.sort(orderedLoot, function(a, b) -- awsome loot sort function :P
			local a_name, _, a_quality, a_iLevel, _, _, a_subclass, _, a_equipSlot = GetItemInfo(a)
			local b_name, _, b_quality, b_iLevel, _, _, b_subclass, _, b_equipSlot = GetItemInfo(b)
			if a_name and b_name then
				if a_quality ~= b_quality then
					return a_quality > b_quality
				elseif a_iLevel ~= b_iLevel then
					return a_iLevel > b_iLevel
				elseif a_subclass ~= b_subclass then
					return a_subclass < b_subclass
				elseif a_equipSlot ~= b_equipSlot then
					return a_equipSlot < b_equipSlot
				end
			elseif a_name or b_name then
				return a_name and true or false
			end
			local aR, bR = loot[i][a], loot[i][b]
			if type(aR) == 'boolean' then
				aR = a
			end
			if type(bR) == 'boolean' then
				bR = b
			end
			return tonumber(aR) > tonumber(bR)
		end)
		for _, item in ipairs(orderedLoot) do
			local btn = AceGUI:Create("YssInstanceMapsLoot")
			btn:SetItem(item, loot[i][item])
			-- Add the button to the container
			btn:SetCallback("OnClick", LootClickHandler)
			btn:SetCallback("OnEnter", LootEnterHandler)
			btn:SetCallback("OnLeave", LootLeaveHandler)
			YssInstanceMapsLootFrame.contentframes[i]:AddChild(btn)
		end
	end
	if numLootTables > #YssInstanceMapsLootFrame.frames then
		for i=numLootTables, #YssInstanceMapsLootFrame.frames do
			YssInstanceMapsLootFrame.frames[i].frame:Hide()
		end
	end

	LayoutLootFrames(YssInstanceMapsLootFrame.frames, numLootTables)
	YssInstanceMapsLootFrame.LootHide:Stop()
	YssInstanceMapsLootFrame.LootShow:Play()
end

local function SetupBossFrame(self, InstanceType, Instance, Boss)
	local text = BB[Boss] or Boss
	self.name:SetText(text)
	self.InstanceType = InstanceType
	self.Instance = Instance
	self.Boss = Boss
end

local function setupBossAnimation(frame)
	local enlargeGroup = frame:CreateAnimationGroup()
	local enlarge = enlargeGroup:CreateAnimation("Animation")
		enlarge:SetDuration(.25)
		enlarge:SetOrder(1)
		enlarge:SetScript("OnPlay", function(self)
			frame:SetScript('OnSizeChanged', nil)
		end)
		enlarge:SetScript("OnUpdate", function(self)
			local progress = self:GetProgress()
			local newSize = bossframesize + (bossframesize * progress)
			frame:SetWidth(newSize)
			frame:SetHeight(newSize)
			frame.SkullBG:SetWidth(newSize*1.5)
			frame.SkullBG:SetHeight(newSize*2)
			if frame.Skull then
				frame.Skull:SetWidth(newSize/1.3)
				frame.Skull:SetHeight(newSize)
			end
			local fontName, fontHeight, fontFlags = frame.name:GetFont()
			frame.name:SetFont(fontName, bossfontsize + bossfontsize * progress, fontFlags)
		end)
		enlarge:SetScript("OnFinished", function(self)
			local newSize = bossframesize * 2
			frame:SetWidth(newSize)
			frame:SetHeight(newSize)
			frame.SkullBG:SetWidth(newSize*1.5)
			frame.SkullBG:SetHeight(newSize*2)
			if frame.Skull then
				frame.Skull:SetWidth(newSize/1.3)
				frame.Skull:SetHeight(newSize)
			end
			local fontName, fontHeight, fontFlags = frame.name:GetFont()
			frame.name:SetFont(fontName, bossfontsize*  2, fontFlags)
		end)
	enlargeGroup.enlarge = enlarge
	enlargeGroup:SetLooping("NONE")
	frame.enlargeGroup = enlargeGroup

	local reduceGroup = frame:CreateAnimationGroup()
	local reduce = reduceGroup:CreateAnimation("Animation")
		reduce:SetDuration(.25)
		reduce:SetOrder(1)
		reduce:SetScript("OnUpdate", function(self)
			local progress = self:GetProgress()
			local newSize = bossframesize + (bossframesize * (1 - progress))
			frame:SetWidth(newSize)
			frame:SetHeight(newSize)
			frame.SkullBG:SetWidth(newSize*1.5)
			frame.SkullBG:SetHeight(newSize*2)
			if frame.Skull then
				frame.Skull:SetWidth(newSize/1.3)
				frame.Skull:SetHeight(newSize)
			end
			local fontName, fontHeight, fontFlags = frame.name:GetFont()
			frame.name:SetFont(fontName, bossfontsize + bossfontsize * (1 - progress), fontFlags)
		end)
		reduce:SetScript("OnFinished", function(self)
			frame:SetWidth(bossframesize)
			frame:SetHeight(bossframesize)
			frame.SkullBG:SetWidth(bossframesize*1.5)
			frame.SkullBG:SetHeight(bossframesize*2)
			if frame.Skull then
				frame.Skull:SetWidth(bossframesize/1.3)
				frame.Skull:SetHeight(bossframesize)
			end
			local fontName, fontHeight, fontFlags = frame.name:GetFont()
			frame.name:SetFont(fontName, bossfontsize, fontFlags)
			frame:SetScript('OnSizeChanged', function() frame:SetScript('OnSizeChanged', BossFrameOnShow) end) -- reenable the OnSizeChanged script on the next OnSizeChanged otherwise we catch the last one frome the animation
		end)
	reduceGroup.reduce = reduce
	reduceGroup:SetLooping("NONE")
	frame.reduceGroup = reduceGroup
	frame:EnableMouse(true)
	frame:SetScript('OnEnter', function(self)
		self.reduceGroup.reduce:Stop()
		self.enlargeGroup.enlarge:Play()
	end)
	frame:SetScript('OnLeave', function(self)
		self.enlargeGroup.enlarge:Stop()
		self.reduceGroup.reduce:Play()
	end)

	local rotateBGGroup = frame.SkullBG:CreateAnimationGroup()
	local rotate = rotateBGGroup:CreateAnimation("Animation")
		rotate:SetDuration(5)
		rotate:SetOrder(1)
		rotate:SetScript("OnUpdate", function(self)
			local angle = (1-self:GetProgress())*2*pi
			local s = sin(angle);
			local c = cos(angle);
			frame.SkullBG:SetTexCoord(	0.5-s, 0.5+c,
						0.5+c, 0.5+s,
						0.5-c, 0.5-s,
						0.5+s, 0.5-c);
		end)
	rotateBGGroup.rotate = rotate
	rotateBGGroup:SetLooping("REPEAT")
	frame.rotateBGGroup = rotateBGGroup
end

local function getBossFrame(parent)
	local frame
	if next(bossframecache) then
		frame = bossframecache[#bossframecache]
		bossframecache[#bossframecache] = nil
	else
		totalFramesGenerated = totalFramesGenerated + 1
		frame = CreateFrame("DressUpModel", "YssInstanceMapsBossFrame"..totalFramesGenerated)
		frame:Hide()
		frame:SetWidth(bossframesize)
		frame:SetHeight(bossframesize)
		frame:SetToplevel(true)
		frame:SetCreature(cid)
		local SkullBG = frame:CreateTexture(nil, 'BACKGROUND')
			SkullBG:SetPoint("CENTER")
			SkullBG:SetWidth(bossframesize*1.5)
			SkullBG:SetHeight(bossframesize*2)
			SkullBG:SetTexture("Interface\\Addons\\YssInstanceMaps\\Art\\SkullBG")
			SkullBG:SetAlpha(.75)
		frame.SkullBG = SkullBG

		local name = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
			local font, size = name:GetFont()
			name:SetFont(font,size,"OUTLINE")
			name:SetHeight(15)
			name:SetPoint("BOTTOM", frame, "TOP")
			name:SetText("Boss Name Here")
		frame.name = name

		if cid == 0 then
			local Skull = frame:CreateTexture(nil, 'OVERLAY')
				Skull:SetPoint("CENTER")
				Skull:SetWidth(bossframesize/1.3)
				Skull:SetHeight(bossframesize)
				Skull:SetTexture("Interface\\WorldMap\\Skull_64")
				Skull:SetTexCoord(0,.5,0,.5)
				Skull:SetAlpha(.75)
			frame.Skull = Skull
		end
		frame:SetScript('OnShow', BossFrameOnShow)
		frame:SetScript('OnMouseUp', BossFrameOnMouseUp)
		frame.SetupBossFrame = SetupBossFrame
		setupBossAnimation(frame)
	end
	frame:SetParent(parent)
	frame:SetScript('OnSizeChanged', BossFrameOnShow) -- fix model for new size/scale
	frame:SetFrameLevel(parent:GetFrameLevel()+10)
	frame.onupdate = 0
	frame:Show()
	return frame
end

local function returnBossFrame(frame)
	frame.name:SetText("Boss Name Here")
	frame.rotateBGGroup:Stop()

	if cid > 0 and frame.Skull then
		frame.Skull:Hide()
		frame.Skull = nil
	end

	frame:Hide()
	frame:ClearAllPoints()
	frame:SetWidth(bossframesize)
	frame:SetHeight(bossframesize)
	frame.SkullBG:SetWidth(bossframesize*1.5)
	frame.SkullBG:SetHeight(bossframesize*2)
	local fontName, fontHeight, fontFlags = frame.name:GetFont()
	frame.name:SetFont(fontName, bossfontsize, fontFlags)
	bossframecache[#bossframecache+1] = frame
	return nil
end

-- /script YssInstanceMaps:AddBosses('Dungeons', 'Gundrak', 1)
function YssInstanceMaps:AddBosses(MapType, Map, level, parent)
	self:ClearBosses(parent)
	if self.Bosses[MapType] and self.Bosses[MapType][Map] then
		local w, h = parent:GetWidth(), parent:GetHeight()*-1
		local bosstable = self.Bosses[MapType][Map]
		for boss, locations in pairs(bosstable) do
			for l,x,y in string.gmatch(locations, "([^|]+):([^|]+):([^|]+)") do
				if tonumber(l) == level then
					local frame = getBossFrame(parent)
					frame:SetPoint("CENTER", parent, "TOPLEFT", tonumber(x)/10000*w, tonumber(y)/10000*h)
					frame:SetupBossFrame(MapType, Map, boss)
					parent.displayedBosses[boss] = frame
				end
			end
		end
	end
	if at:IsBoss(MapType, Map, "Trash Mobs") then
		local frame = getBossFrame(parent)
		frame:SetPoint("RIGHT", parent, "RIGHT", bossframesize*-1.5, 0)
		--frame.name:SetText(BB[boss])
		frame:SetupBossFrame(MapType, Map, "Trash Mobs")
		parent.displayedBosses["Trash Mobs"] = frame
	end
end

function YssInstanceMaps:ClearBosses(parent)
	if parent.displayedBosses then
		for b, frame in pairs(parent.displayedBosses) do
			parent.displayedBosses[b] = returnBossFrame(frame)
		end
	else
		parent.displayedBosses = {}
	end
end

YssInstanceMaps.DropDownMenu = CreateFrame("Frame", "YssInstanceMaps_DropDownMenu", YssInstanceMaps)
YssInstanceMaps.DropDownMenu.onHide = function(...)
	MenuParent = nil
	MenuItem = nil
	MenuEquipSlot = nil
end

--FLOOR = "Areas";
--FLOOR_NUMBER = "Area %d";
--local dropdowntype = nil --'type', 'zone', 'level'
local info = {}
YssInstanceMaps.DropDownMenu.initialize = function(self, level)
	if not level then return end
	wipe(info)
	if dropdowntype == 'type' then
		local names = {}
		local RLnames = {}
		for name, Lname in pairs(YssInstanceMaps.MapTypes) do
			names[#names+1] = Lname
			RLnames[Lname] = name
		end
		table.sort(names)

		for i = 1, #names do
			info.text = names[i]
			info.arg1 = RLnames[names[i]]
			info.arg2 = names[i]
			info.func = function(button, arg1, arg2)
				if arg1 ~= YssInstanceMaps.db.profile.maptype then
					YssInstanceMaps.db.profile.maptype = arg1
					YssInstanceMaps:SetMap(nil)
					YssInstanceMapsTypeDropdown.text:SetText(arg2)
					YssInstanceMaps.db.profile.zone = nil
					YssInstanceMapsZoneDropdown.text:SetText('')
				end
				YssInstanceMapsLevelDropdown:Hide()
			end
			info.checked = RLnames[names[i]] == YssInstanceMaps.db.profile.maptype
			UIDropDownMenu_AddButton(info, level)
		end
	elseif dropdowntype == 'zone' then
		local names = {}
		local RLnames = {}
		for name, tex in pairs(YssInstanceMaps.Maps[YssInstanceMaps.db.profile.maptype]) do
			names[#names+1] = BZ[name] or name
			RLnames[BZ[name] or name] = name
		end
		table.sort(names)

		for i = 1, #names do
			info.text = names[i]
			info.arg1 = RLnames[names[i]]
			info.arg2 = names[i]
			info.func = function(button, arg1, arg2)
				YssInstanceMaps.db.profile.zone = arg1
				---- need alot more logic here to get the first map since not all start at 1
				local baselevel, numlevels = YssInstanceMaps:GetLowestLevel(YssInstanceMaps.db.profile.maptype, arg1)
				YssInstanceMaps:SetMap(YssInstanceMaps.Maps[YssInstanceMaps.db.profile.maptype][arg1], baselevel)
				YssInstanceMaps:AddBosses(YssInstanceMaps.db.profile.maptype, arg1, baselevel, YssInstanceMapsDetailFrame)
				YssInstanceMaps.db.profile.level = baselevel
				YssInstanceMapsZoneDropdown.text:SetText(arg2)
				if numlevels then
					YssInstanceMapsLevelDropdown:Show()
					YssInstanceMapsLevelDropdown.text:SetText(YssInstanceMaps:GetLevelName(YssInstanceMaps.db.profile.maptype, arg1, baselevel))
				else
					YssInstanceMapsLevelDropdown:Hide()
				end
			end
			info.checked = RLnames[names[i]] == YssInstanceMaps.db.profile.zone
			UIDropDownMenu_AddButton(info, level)
		end
	elseif dropdowntype == 'level' then
		local MapType = YssInstanceMaps.db.profile.maptype
		local Map = YssInstanceMaps.db.profile.zone
		local levels = YssInstanceMaps.Levels[MapType][Map]
		local leveltable
		if type(levels) == 'table' then
			leveltable = levels
		else
			leveltable = {}
			for i=1, levels do
				leveltable[i] = i
			end
		end
		local levelsnames = {}
		for i=1, #leveltable do
			levelsnames[i] = YssInstanceMaps:GetLevelName(MapType, Map, leveltable[i])
		end
		for i = 1, #leveltable do
			info.text = levelsnames[i]
			info.arg1 = leveltable[i]
			info.arg2 = levelsnames[i]
			info.func = function(button, arg1, arg2)
				YssInstanceMaps:SetMap(YssInstanceMaps.Maps[MapType][Map], arg1)
				YssInstanceMaps:AddBosses(MapType, Map, arg1, YssInstanceMapsDetailFrame)
				YssInstanceMaps.db.profile.level = arg1
				YssInstanceMapsLevelDropdown.text:SetText(arg2)
			end
			info.checked = leveltable[i] == YssInstanceMaps.db.profile.level
			UIDropDownMenu_AddButton(info, level)
		end
	end
	DropDownList1:SetScale(YssInstanceMaps.db.profile.frame.scale)
end
local function dropdownClick(self)
	dropdowntype = self.type
	ToggleDropDownMenu(1, nil, YssInstanceMaps_DropDownMenu, self:GetParent(),11,7)
end
YssInstanceMapsTypeDropdown.button:SetScript('OnClick', dropdownClick)
YssInstanceMapsZoneDropdown.button:SetScript('OnClick', dropdownClick)
YssInstanceMapsLevelDropdown.button:SetScript('OnClick', dropdownClick)
